package cn.qqck.kotlin.tools

import java.io.File
import java.nio.ByteBuffer
import java.nio.channels.FileChannel
import java.nio.file.Paths
import java.nio.file.StandardOpenOption

/**
 * 将字符串内容写入文件。
 *
 * @param name 文件名称。
 * @param dir 文件路径，默认为空，表示当前路径。
 * @param append 是否以追加模式写入，默认为 false 表示覆盖写入。
 * @return 写入是否成功，成功返回 true，否则返回 false。
 */
fun String.fset(name: String, dir: String? = null, append: Boolean = false): Boolean = this.toByteArray().fset(name, dir, append)

/**
 * 将当前 ByteArray 的内容写入到指定文件。
 *
 * @param name 文件名，当提供 `dir` 参数时，此参数作为文件的相对路径或者文件名；如果 `dir` 未指定，则此参数为完整的文件路径。
 * @param dir 可选的目录路径。如果指定，将 `name` 作为子路径结合形成完整文件路径。
 * @param append 如果为 `true`，数据将追加到文件末尾；如果为 `false`，将覆盖文件内容。
 * @return 返回布尔值，表示文件写入操作是否成功。
 */
fun ByteArray.fset(name: String, dir: String? = null, append: Boolean = false): Boolean {
    val j_name = if (dir == null) Paths.get(name) else Paths.get(dir, name)
    do {
        try {
            val j_c = FileChannel.open(j_name, StandardOpenOption.CREATE, if (append) StandardOpenOption.APPEND else StandardOpenOption.WRITE)
            do {
                try {
                    val j_lock = j_c.lock()
                    try {
                        j_c.write(ByteBuffer.wrap(this))
                        if (!append) j_c.truncate(j_c.position())
                        return true
                    } catch (_: Exception) {
                        return false
                    } finally {
                        j_lock.close()
                        j_c.close()
                    }
                } catch (_: java.nio.channels.OverlappingFileLockException) {
                    Thread.sleep(1)
                } catch (_: Exception) {
                    j_c.close()
                    return false
                }
            } while (true)
        } catch (e: java.nio.file.NoSuchFileException) {
            if (dir == null || !File(dir).mkdirs()) return false
        } catch (_: Exception) {
            return false
        }
    } while (true)
}

/**
 * 从当前字符串表示的文件路径中读取内容并返回其字节数组。
 * 如果文件使用过程中出现异常、大小超出限制或读取失败，则返回 null。
 *
 * @return 文件内容的字节数组，若读取失败则返回 null。
 */
fun String.fget(): ByteArray? {
    try {
        val j_c = FileChannel.open(Paths.get(this), StandardOpenOption.WRITE, StandardOpenOption.READ)
        do {
            try {
                val j_lock = j_c.lock()
                try {
                    val j_size = j_c.size()
                    if (j_size > Int.MAX_VALUE) return null
                    val j_buf = ByteBuffer.allocate(j_size.toInt())
                    if (j_c.read(j_buf) != j_size.toInt()) return null
                    return j_buf.array()
                } catch (_: Exception) {
                    return null
                } finally {
                    j_lock.close()
                    j_c.close()
                }
            } catch (_: java.nio.channels.OverlappingFileLockException) {
                Thread.sleep(1)
            } catch (e: Exception) {
                j_c.close()
                return null
            }
        } while (true)
    } catch (_: Exception) {
    }
    return null
}
