package cn.qqck.kotlin.tools

fun String.bytes() = this.toByteArray()
fun String.bool(): Boolean? {
    if (this == "1" || this == "t" || this == "T" || this == "true" || this == "True" || this == "yes" || this == "Yes") return true
    if (this == "0" || this == "f" || this == "F" || this == "false" || this == "False" || this == "no" || this == "No") return false
    return null
}

/**
 * 从当前字符串中获取指定子字符串前的内容。
 *
 * @param find 要查找的子字符串。如果子字符串不存在，返回值将为 null。
 * @return 当前字符串中位于指定子字符串之前的部分。如果子字符串未找到，返回 null。
 */
fun String.left(find: String): String? {
    val j_index = this.indexOf(find)
    if (j_index == -1) return null
    return this.substring(0, j_index)
}

/**
 * 返回从指定字符串 `find` 的出现位置开始一直到目标字符串末尾的子字符串。
 * 如果目标字符串中未找到 `find`，返回 `null`。
 *
 * @param find 要查找的字符串。
 * @return 如果找到，则返回从 `find` 结束位置到目标字符串末尾的子字符串；否则返回 `null`。
 */
fun String.right(find: String): String? {
    val j_index = this.indexOf(find)
    if (j_index == -1) return null
    return this.substring(j_index + find.length, this.length)
}

/**
 * 统计子字符串在当前字符串中出现的次数。
 *
 * @param sub 要统计的子字符串。
 * @return 子字符串在当前字符串中出现的次数。如果子字符串为空或不存在，则返回0。
 */
fun String.count(sub: String): Int {
    var j_count = 0
    var j_index = this.indexOf(sub)
    while (j_index > -1) {
        j_count++
        j_index = this.indexOf(sub, j_index + sub.length)
    }
    return j_count
}

/**
 * 获取当前字符串中从指定左边界字符串到右边界字符串之间的子字符串。
 *
 * @param left 左边界字符串，如果找不到该字符串则返回null。
 * @param right 右边界字符串，如果找不到该字符串则返回null。
 * @return 位于左边界字符串之后，右边界字符串之前的子字符串。
 *         如果任意边界字符串未找到，返回null。
 */
fun String.sub(left: String, right: String): String? {
    var j_left = this.indexOf(left)
    if (j_left == -1) return null
    j_left += left.length
    val j_right = this.indexOf(right, j_left)
    if (j_right == -1) return null
    return this.substring(j_left, j_right)
}

/**
 * 判断当前字符串是否仅包含数字字符（0-9）。
 *
 * @return 如果字符串不为空且仅包含数字字符，则返回true；否则返回false。
 */
fun String.is09(): Boolean {
    if (this.isEmpty()) return false
    return this.all { it.isDigit() }
}

/**
 * 判断当前字符串是否为有效的十六进制字符串。
 *
 * 十六进制字符串的定义为仅包含以下字符：'0'-'9'、'a'-'f' 或 'A'-'F'。
 *
 * @return 如果字符串为有效的十六进制字符串，返回 true；否则返回 false。
 */
fun String.isHex(): Boolean {
    if (this.isEmpty()) return false
    return this.all { it.isDigit() || (it in 'a'..'f') || (it in 'A'..'F') }
}

/**
 * 检查当前字符串是否为合法的十六进制字符串。
 * 字符串中的每个字符必须是数字（0-9）或小写字母（a-f）。
 *
 * @return 如果字符串为合法的十六进制字符串，返回 true；否则返回 false。
 */
fun String.ishex(): Boolean {
    if (this.isEmpty()) return false
    return this.all { it.isDigit() || (it in 'a'..'f') }
}

/**
 * 判断当前字符串是否为有效的十六进制字符串。
 *
 * 此方法会验证字符串中是否仅包含合法的十六进制字符（0-9，A-F）。
 *
 * @return 如果字符串是有效的十六进制字符串则返回 true，否则返回 false。
 */
fun String.isHEX(): Boolean {
    if (this.isEmpty()) return false
    return this.all { it.isDigit() || (it in 'A'..'F') }
}

/**
 * 判断当前字符串是否全部由 ASCII 可打印字符组成（从 '!' 到 '~' 的范围）。
 *
 * @return 如果字符串非空且全部由 ASCII 可打印字符组成，返回 true；否则返回 false。
 */
fun String.isEx(): Boolean {
    if (this.isEmpty()) return false
    return this.all { it in '!'..'~' }
}

data class AddrPort(val addr: String, val port: Int)

/**
 * 将当前字符串解析为主机地址和端口号，返回一个包含地址和端口的 AddrPort 对象。
 * 支持的格式包括：
 * - IPv6地址:端口
 * - 主机名或IPv4地址:端口
 *
 * @return 如果解析成功，返回包含地址和端口的 AddrPort 对象；如果解析失败或格式无效，则返回 null。
 */
fun String.toAddrPort(): AddrPort? {
    if (this.isBlank()) return null
    Regex("\\[([a-fA-F0-9:]+)]:(\\d+)").matchEntire(this)?.also {
        val j_port = it.groupValues[2].toIntOrNull()
        if (j_port != null && j_port in 1..65535) return AddrPort(it.groupValues[1], j_port)
    }
    Regex("([a-zA-Z0-9.-]+):(\\d+)").matchEntire(this)?.also {
        val j_port = it.groupValues[2].toIntOrNull()
        if (j_port != null && j_port in 1..65535) return AddrPort(it.groupValues[1], j_port)
    }
    return null
}

/**
 * 隐藏字符串中间一部分字符，用于部分信息的脱敏显示。
 *
 * @param hs 替代被隐藏字符的字符，默认为 '*'。
 * @param pre_len 保留的前缀字符个数，默认为 2。
 * @param suf_len 保留的后缀字符个数，默认为 2。
 * @return 隐藏中间部分字符后的字符串。当字符串长度小于等于前缀长度和后缀长度之和时，原样返回字符串。
 */
fun String.hide(hs: Char = '*', pre_len: Int = 2, suf_len: Int = 2): String {
    val j_slen = this.length
    if (j_slen <= pre_len + suf_len) return this
    val j_midlen = j_slen - pre_len - suf_len
    val j_shide = CharArray(j_slen)
    for (j_i in 0 until pre_len) j_shide[j_i] = this[j_i]
    for (j_i in pre_len until pre_len + j_midlen) j_shide[j_i] = hs
    for (j_i in j_slen - suf_len until j_slen) j_shide[j_i] = this[j_i]
    return String(j_shide)
}
